from server import message_handler, get_address, get_port
from common.utils import read_conf
from unittest import TestCase, main
from time import ctime
from sys import argv
from common.config_path_file import CONFIG_PATH


class TestServer(TestCase):
    """Тестовый класс для тестирования функций сервера"""
    config = read_conf(CONFIG_PATH)
    test_dict_get_ok = {
        config['RESPONSE']: 200
    }
    test_dict_get_err = {
        config['RESPONSE']: 400,
        config['ERROR']: 'Bad Request'
    }
    test_dict_send = {
        config['ACTION']: config['PRESENCE'],
        config['TIME']: ctime(),
        config['USER_NAME']: {
            config['ACCOUNT_NAME']: 'Test_name'
        }
    }

    def test_message_handler_ok(self):
        """Тестирует если пришёл не правильный словарь в сообщении"""
        self.assertEqual(message_handler(self.test_dict_get_err, CONFIG_PATH), self.test_dict_get_err)

    def test_get_address_ok(self):
        """Тестирует правильный ли адрес вернёт функция get_address()"""
        argv.clear()
        argv.append('/snap/pycharm-community/276/plugins/python-ce/helpers/pycharm/_jb_unittest_runner.py')
        argv.append('server.py')
        argv.append('-a')
        argv.append(self.config['ADDR_DEF'])
        self.assertEqual(get_address(CONFIG_PATH), self.config['ADDR_DEF'])

    def test_get_address_none_port(self):
        """Тестирует вернётся ли None если не указать порт в аргументах"""
        argv.clear()
        argv.append('/snap/pycharm-community/276/plugins/python-ce/helpers/pycharm/_jb_unittest_runner.py')
        argv.append('server.py')
        argv.append('-a')
        self.assertIsNone(get_address(CONFIG_PATH), None)

    def test_get_port_type(self):
        """Тестирует правильный ли тип данных вернёт функция get_port()"""
        argv.clear()
        argv.append('/snap/pycharm-community/276/plugins/python-ce/helpers/pycharm/_jb_unittest_runner.py')
        argv.append('-p')
        argv.append('7778')
        self.assertIsInstance(get_port(CONFIG_PATH), int)


if __name__ == '__main__':
    main()



from dis import get_instructions


class ServerVerifier(type):
    def __init__(cls, meta_name, bases_classes, meta_dict):
        methods_glob = list()

        for key in meta_dict.keys():
            try:
                iterator = get_instructions(meta_dict[key])
            except (TypeError, SyntaxError):
                pass
            # разбор данных что в итераторе
            else:
                for item in iterator:
                    if item.opname == 'LOAD_GLOBAL':
                        if item.argval not in methods_glob:
                            methods_glob.append(item.argval)  # SOCK_STREAM и AF_INET у меня в LOAD_GLOBAL

        if 'connect' in methods_glob:
            raise TypeError('Метод connect не может быть использован в классе сервер')
        if not ('SOCK_STREAM' in methods_glob and 'AF_INET' in methods_glob):
            raise TypeError('Сокет был неправильно инициализирован!')
        super().__init__(meta_name, bases_classes, meta_dict)


class ClientVerifier(type):
    def __init__(cls, meta_name, bases_classes, meta_dict):
        methods_glob = list()

        for key in meta_dict.keys():
            try:
                iterator = get_instructions(meta_dict[key])
            except (TypeError, SyntaxError):
                pass
            # разбор данных что в итераторе
            else:
                for item in iterator:
                    if item.opname == 'LOAD_GLOBAL':
                        if item.argval not in methods_glob:
                            methods_glob.append(item.argval)

        if 'accept' in methods_glob or 'listen' in methods_glob:
            raise TypeError('В классе используется запрещённый метод!')
        if 'get_message' in methods_glob or 'send_message' in methods_glob:
            pass
        else:
            raise TypeError('Отсутствует вызовы функций для работы с сокетами!')
        super().__init__(meta_name, bases_classes, meta_dict)



from unittest import TestCase, main
from common.utils import read_conf, get_message, send_message
from json import dumps
from time import ctime
from common.config_path_file import CONFIG_PATH


class TestSocket:
    """Тестовый класс для тестирования отправки и получения сообщений"""
    def __init__(self, test_dict):
        self.test_dict = test_dict
        self.encode_message = None
        self.get_message = None
        self.config = read_conf(CONFIG_PATH)

    def send(self, message):
        """Отправляет корректно закодированное сообщение"""
        test_message = dumps(self.test_dict)  # превращаем словарь в строку
        self.encode_message = test_message.encode(self.config['ENCODING'])  # кодируем данные
        self.get_message = message

    def recv(self, max_length):
        """Получает данные из сокета"""
        test_message = dumps(self.test_dict)
        return test_message.encode(self.config['ENCODING'])


class TestUtils(TestCase):
    """Тестовый класс для тестирования функций: read_conf(), которая читает файл config.yaml.,
    send_message(), которая кодирует и отправляет сообщение, get_message(), которая получает и декодирует сообщение"""

    config = read_conf(CONFIG_PATH)
    test_dict_get_ok = {
        config['RESPONSE']: 200
    }
    test_dict_get_err = {
        config['RESPONSE']: 400,
        config['ERROR']: 'Bad request!'
    }
    test_dict_send = {
        config['ACTION']: config['PRESENCE'],
        config['TIME']: ctime(),
        config['USER_NAME']: {
            config['ACCOUNT_NAME']: 'Test_name'
        }
    }
    conf_dict = {
        'ADDR_LISTEN_DEF': '',
        'ADDR_SERV': 'addr_server',
        'ADDR_DEF': '127.0.0.1',
        'PORT_DEF': 7777,
        'PORT_SERV': 'port_server',
        'MAX_CONNECT': 6,
        'MAX_MESSAGE_LEN_BYTE': 1024,
        'ENCODING': 'utf-8',
        'ACTION': 'action',
        'TIME': 'time',
        'USER_NAME': 'user',
        'ACCOUNT_NAME': 'account_name',
        'PRESENCE': 'presence',
        'RESPONSE': 'response',
        'ERROR': 'error'
    }

    def test_read_conf(self):
        """Тестирует возращает ли данные функция read_conf()"""
        self.assertTrue(self.config)

    def test_read_conf_2(self):
        """Проверяет правильные ли данные возращает функция read_conf()"""
        self.assertEqual(self.config, self.conf_dict)

    def test_send_message_true(self):
        """Тестирует корректность отправки сообщения"""
        test_socket = TestSocket(self.test_dict_send)
        send_message(test_socket, self.test_dict_send, CONFIG_PATH)
        self.assertEqual(test_socket.encode_message, test_socket.get_message)

    def test_send_message_error(self):
        """Тестирует сообщение с неправильным типом данных"""
        test_socket = TestSocket(self.test_dict_send)
        send_message(test_socket, self.test_dict_send, CONFIG_PATH)
        self.assertRaises(TypeError, test_socket.test_dict, test_socket.get_message, 'error_dictionary!')

    def test_send_message_bytes(self):
        """Проверяет имеет ли сообщение тип данных bytes"""
        test_socket = TestSocket(self.test_dict_send)
        send_message(test_socket, self.test_dict_send, CONFIG_PATH)
        self.assertIsInstance(test_socket.encode_message, bytes)

    def test_get_message_ok(self):
        """Тест приёма сообщения (словаря) на корректность содержимого"""
        test_sock_ok = TestSocket(self.test_dict_get_ok)
        self.assertEqual(get_message(test_sock_ok, CONFIG_PATH), self.test_dict_get_ok)

    def test_get_message_error(self):
        """Тест приёма сообщения c неправильным словарём"""
        test_sock_err = TestSocket(self.test_dict_get_err)
        self.assertEqual(get_message(test_sock_err, CONFIG_PATH), self.test_dict_get_err)

    def test_get_message_key_resp(self):
        """Проверка на содержание части строки в названии ключа словаря из сообщения"""
        test_sock_ok = TestSocket(self.test_dict_get_ok)
        message = get_message(test_sock_ok, CONFIG_PATH)
        key_name = None
        for key in message.keys():
            if 'resp' in key:
                key_name = key
        self.assertIsNotNone(key_name)


if __name__ == '__main__':
    main()



import hmac
from binascii import hexlify, b2a_base64
from hashlib import pbkdf2_hmac
from json import JSONDecodeError
from socket import socket, AF_INET, SOCK_STREAM
from sys import path
path.append('../')
from time import ctime, sleep
from common.utils import read_conf, send_message, get_message
from common.decorators import log
from threading import Thread, Lock
from logging import getLogger
from PyQt5.QtCore import QObject, pyqtSignal
from common.erros import ServerError
from common.config_path_file import CONFIG_PATH

client_logger = getLogger('client')
sock_lock = Lock()
conf = read_conf(CONFIG_PATH)


class ClientTransport(Thread, QObject):
    """Класс для взаимодействия клиента с сервером"""
    # cигналы: новое сообщение и потеря соединения
    new_message = pyqtSignal(dict)
    connect_lost = pyqtSignal()
    mess_205 = pyqtSignal()

    def __init__(self, port_server, addr_server, data_base, name_client, password, keys_encrypt):
        Thread.__init__(self)
        QObject.__init__(self)
        self.data_base = data_base
        self.name_client = name_client
        self.transport = None  # сокет для работы с серваком
        self.password = password
        self.keys_encrypt = keys_encrypt
        self.connect_init(port_server, addr_server)

        # обновление таблиц известных пользователей и контактов
        try:
            self.user_list_request()
            self.contacts_list_request()
        except OSError as err:
            if err.errno:
                client_logger.critical('Соединение с сервером было потеряно!')
                raise ServerError('Соединение с сервером было потеряно!')
            client_logger.error('Timeout соединения при обновлении списка пользователей!')
        except JSONDecodeError:
            client_logger.critical('Соединение с сервером было потеряно!')
            raise ServerError('Соединение с сервером было потеряно!')
        # флаг для продолжения работы транспорта
        self.running_flag = True

    @log
    def create_welcome_message(self):
        """Метод класса вернёт приветственное сообщение"""
        return {
            conf['ACTION']: conf['PRESENCE'],
            conf['TIME']: ctime(),
            conf['USER_NAME']: {
                conf['ACCOUNT_NAME']: self.name_client,
                conf['PUB_KEY']: None
            }
        }

    @log
    def create_out_message(self):
        """Метод класса возвращает словарь с сообщением о выходе"""
        return {
            conf['ACTION']: conf['OUT'],
            conf['TIME']: ctime(),
            conf['ACCOUNT_NAME']: self.name_client
        }

    @log
    def response_analysis(self, message):
        """Метод класса выполняющий разбор ответа сервера"""
        client_logger.debug(f'Выполняется разбор ответа сервера - {message}')
        if conf['RESPONSE'] in message:
            if message[conf['RESPONSE']] == 200:
                client_logger.info('Получен ответ от сервера - 200 : OK')
                return
            elif message[conf['RESPONSE']] == 400:
                client_logger.error(f'Получен ответ с ошибкой от сервера - 400 : {message[conf["ERROR"]]}')
                raise ServerError(f'{message[conf["ERROR"]]}')
            elif message[conf['RESPONSE']] == 205:
                client_logger.debug('Получен запрос на изменение данных.')
                self.user_list_request()
                self.contacts_list_request()
                self.mess_205.emit()
            else:
                client_logger.debug(f'Получен не известный код {message[conf["RESPONSE"]]}!')
        # если это сообщение от пользователя добавляем в базу и дадим сигнал о новом сообщении
        elif conf['ACTION'] in message and message[conf['ACTION']] == conf['MESSAGE'] and conf['ADDRESSER'] in message \
                and conf['TARGET'] in message and conf['MESS_TEXT'] in message \
                and message[conf['TARGET']] == self.name_client:
            client_logger.debug(f'Получено сообщение от пользователя {message[conf["ADDRESSER"]]}:'
                                f'{message[conf["MESS_TEXT"]]}')
            self.new_message.emit(message)

    @log
    def connect_init(self, port_server, addr_server):
        """Метод класса для инициализации соединения с сервером"""
        self.transport = socket(AF_INET, SOCK_STREAM)
        # таймаут для освобождения сокета
        self.transport.settimeout(6)

        # пытаемся соединиться 6 попыток, флаг успеха ставим в True если удалось
        connect_flag = False
        for i in range(6):
            client_logger.info(f'Номер попытки подключения - {i + 1}')
            try:
                self.transport.connect((addr_server, port_server))
            except (OSError, ConnectionRefusedError, ConnectionError):
                pass
            else:
                connect_flag = True
                client_logger.debug(f'Установлено соединение с сервером {addr_server}:{port_server}')
                break
            sleep(1.2)

        if not connect_flag:
            client_logger.critical(f'Подключение к серверу {addr_server}:{port_server} не удалось!')
            raise ServerError(f'Подключение к серверу {addr_server}:{port_server} не удалось!')

        client_logger.debug(f'Старт диалога аутентификации.')

        # получение хэша пароля
        password_bytes = self.password.encode('utf-8')
        salt = self.name_client.lower().encode('utf-8')
        password_hash = pbkdf2_hmac('sha512', password_bytes, salt, 10000)
        passwd_hash_string = hexlify(password_hash)

        client_logger.debug(f'Хеш пароля готов: {passwd_hash_string}')

        # получение публичного ключа и декодирование его из байт
        pub_key = self.keys_encrypt.publickey().export_key().decode('ascii')

        # авторизация
        with sock_lock:
            req = self.create_welcome_message()
            req[conf['USER_NAME']][conf['PUB_KEY']] = pub_key
            client_logger.debug(f'Приветственное сообщение - {req}')

            # отправка серверу приветственного сообщения
            try:
                send_message(self.transport, req)
                ans = get_message(self.transport)
                client_logger.debug(f'Ответ сервера - {ans}.')

                if conf['RESPONSE'] in ans:
                    if ans[conf['RESPONSE']] == 400:
                        raise ServerError(ans[conf['ERROR']])
                    elif ans[conf['RESPONSE']] == 511:
                        # если всё впорядке авторизация продолжается
                        answer_data = ans[conf['DATA']]
                        hash_passwd = hmac.new(passwd_hash_string, answer_data.encode('utf-8'), 'MD5')
                        digest = hash_passwd.digest()
                        my_answer = conf['RESP_511']
                        my_answer[conf['DATA']] = b2a_base64(digest).decode('ascii')
                        send_message(self.transport, my_answer)
                        self.response_analysis(get_message(self.transport))
            except (OSError, JSONDecodeError):
                client_logger.critical('Потеряно соединение с сервером!')
                raise ServerError('Потеряно соединение с сервером!')

    @log
    def user_list_request(self):
        """Метод класса делает запрос известных пользователей и обновляет их в бд"""
        client_logger.debug(f'Запрос списка известных пользователей {self.name_client}')
        req = {
            conf['ACTION']: conf['GET_USERS'],
            conf['TIME']: ctime(),
            conf['ACCOUNT_NAME']: self.name_client
        }
        with sock_lock:
            send_message(self.transport, req)
            answer = get_message(self.transport)
        if conf['RESPONSE'] in answer and answer[conf['RESPONSE']] == 202:
            self.data_base.add_users(answer[conf['DATA_LIST']])
        else:
            client_logger.error('Список известных пользователей не был обнавлён!')

    @log
    def contacts_list_request(self):
        """Метод класса для запроса листа контактов и их обновления в бд"""
        client_logger.debug(f'Запрос контакт листа для пользователя {self.name_client}')
        req = {
            conf['ACTION']: conf['GET_CONTACTS'],
            conf['TIME']: ctime(),
            conf['USER_NAME']: self.name_client
        }
        client_logger.debug(f'Сформирован запрос {req}')
        with sock_lock:
            send_message(self.transport, req)
            answer = get_message(self.transport)
            client_logger.debug(f'Получен ответ {answer}')
        if conf['RESPONSE'] in answer and answer[conf['RESPONSE']] == 202:
            for cont in answer[conf['DATA_LIST']]:
                self.data_base.add_contact(cont)
        else:
            client_logger.error('Не удалось обновление списка контактов!')

    @log
    def add_contact(self, contact):
        """Метод класса добавляющий пользователя в список контактов"""
        client_logger.debug(f'Попытка создать контакт - {contact}')
        req = {
            conf['ACTION']: conf['ADD_CONTACT'],
            conf['TIME']: ctime(),
            conf['USER_NAME']: self.name_client,
            conf['ACCOUNT_NAME']: contact
        }
        with sock_lock:
            send_message(self.transport, req)
            self.response_analysis(get_message(self.transport))

    @log
    def dell_contact(self, contact):
        """Метод класса для удаления контакта"""
        client_logger.debug(f'Попытка удаления контакта {contact}.')
        req = {
            conf['ACTION']: conf['DEL_CONTACT'],
            conf['TIME']: ctime(),
            conf['USER_NAME']: self.name_client,
            conf['ACCOUNT_NAME']: contact
        }
        with sock_lock:
            send_message(self.transport, req)
            self.response_analysis(get_message(self.transport))

    @log
    def transport_close(self):
        """Метод класса для завершения работы транспорта"""
        self.running_flag = False
        message = self.create_out_message()
        with sock_lock:
            try:
                send_message(self.transport, message)
            except OSError:
                pass
        client_logger.debug('Транспорт выключается.')
        sleep(0.6)

    @log
    def send_message(self, target, message):
        """Метод класса для отправки сообщений"""
        message_dict = {
            conf['ACTION']: conf['MESSAGE'],
            conf['ADDRESSER']: self.name_client,
            conf['TARGET']: target,
            conf['TIME']: ctime(),
            conf['MESS_TEXT']: message
        }
        client_logger.debug(f'Было сформированно сообщение: {message_dict}.')

        with sock_lock:
            send_message(self.transport, message_dict)
            self.response_analysis(get_message(self.transport))
            client_logger.info(f'Сообщение выслано пользователю {target}.')

    @log
    def key_pub_request(self, login):
        """Метод класса для запроса с сервера публичного ключа."""
        client_logger.debug(f'Попытка запроса публичного ключа для - {login}')
        req = {
            conf['ACTION']: conf['GET_PUB_KEY'],
            conf['TIME']: ctime(),
            conf['ACCOUNT_NAME']: login
        }
        with sock_lock:
            send_message(self.transport, req)
            ans = get_message(self.transport)
        if conf['RESPONSE'] in ans and ans[conf['RESPONSE']] == 511:
            return ans[conf['DATA']]
        else:
            client_logger.error(f'Получение ключа собеседника не вышло {login}!')

    @log
    def run(self):
        client_logger.debug('Приёмник сообщений с сервера запущен.')
        while self.running_flag:
            sleep(1.2)
            message = None
            with sock_lock:
                if not self.running_flag:
                    break
                try:
                    self.transport.settimeout(0.6)
                    message = get_message(self.transport)
                except OSError as err:
                    if err.errno:
                        client_logger.critical('Соединение с сервером было потеряно!')
                        self.running_flag = False
                        self.connect_lost.emit()
                except (ConnectionError, ConnectionAbortedError,
                        ConnectionResetError, JSONDecodeError, TypeError):
                    client_logger.debug('Соединение с сервером было потеряно!.')
                    self.running_flag = False
                    self.connect_lost.emit()
                finally:
                    self.transport.settimeout(6)

            # Если получиили сообщение, то вызываем обработчик:
            if message:
                client_logger.debug(f'Принято сообщение с сервера: {message}')
                self.response_analysis(message)



from json import loads, dumps
from yaml import load, FullLoader
from logging import getLogger
from logs import server_log_config, client_log_config
from common.config_path_file import CONFIG_PATH
from sys import argv
from re import search


logger = None
module = argv[0].split('/')[-1]

if 'server' in module:
    logger = getLogger('server')
elif 'client' in module:
    logger = getLogger('client')


def read_conf(file_name):
    """Читает конфигурационный файл в формате yaml"""
    with open(file_name, 'r', encoding='utf-8') as config:
        return load(config, Loader=FullLoader)


def read_conf_lines(file_name):
    """Читает конфегурационный файл и возвращает его в виде списка строк"""
    with open(file_name, 'r', encoding='utf-8') as config:
        return config.readlines()


def replace_value_list_conf(conf_list, key, value):
    """Функция для замены значения ключа кофига. Работает со списком строк (в каждой строке ключ и значение)"""
    for line in conf_list:
        if ':' in line:
            key_value = line.split(':')
            if key_value[0] == key:
                key_value[1] = value + '\n'
                conf_list[conf_list.index(line)] = ': '.join(key_value)
                return conf_list


conf = read_conf(CONFIG_PATH)


def check_gui_argv():
    """Функция для проверки аргумента нужен ли gui при запуске сервера"""
    if '--no_gui' in argv:
        return True
    return False


def check_addr_argv():
    """Функция для проверки аргумента адреса на предмет наличия"""
    try:
        if '-a' in argv:
            return argv[argv.index('-a') + 1]
        elif '--addr' in argv:
            return argv[argv.index('-addr') + 1]
        else:
            return conf['ADDR_DEF']
    except IndexError:
        return conf['ADDR_DEF']


def check_name_argv():
    """Функция для проверки аргумента имени на предмет наличия"""
    logger.debug('Проверка аргумента имени')
    try:
        if '-n' in argv:
            return argv[argv.index('-n') + 1]
        elif '--name' in argv:
            return argv[argv.index('--name') + 1]
        else:
            return None
    except IndexError:
        return None


def check_port_argv():
    """Функция для проверки аргумента порта на предмет наличия """
    logger.debug('Проверка аргумента порта')
    try:
        if '--port' in argv:
            return argv[argv.index('--port') + 1]
        else:
            return conf['PORT_DEF']
    except IndexError:
        return conf['PORT_DEF']


def check_passwd_argv():
    """Функция для проверки аргумента пароля"""
    logger.debug('Проверка аргумента пароля')
    try:
        if '--password' in argv:
            return argv[argv.index('--password') + 1]
        elif '--passwd' in argv:
            return argv[argv.index('--passwd') + 1]
        elif '-p' in argv:
            return argv[argv.index('-p') + 1]
        else:
            return None
    except IndexError:
        return None


def get_message(client):
    """Принимает и декодирует сообщение"""
    encode_response = client.recv(conf['MAX_MESSAGE_LEN_BYTE'])
    if str(type(encode_response)) == "<class 'bytes'>":
        if encode_response == b'':
            return None
        json_response = encode_response.decode(conf['ENCODING'])
        if str(type(json_response)) == "<class 'str'>":
            response = loads(json_response)
            if str(type(response)) == "<class 'dict'>":
                return response
            raise ValueError
        raise ValueError
    raise ValueError


def send_message(sock, message):
    """Кодирует и отправляет сообщение"""
    if str(type(message)) != "<class 'dict'>":
        raise ValueError
    json_message = dumps(message)
    encode_message = json_message.encode(conf['ENCODING'])
    sock.send(encode_message)



from sys import path as sys_path
sys_path.append('../')
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton
from logs import client_log_config
from logging import getLogger


class AddContactDialog(QDialog):
    """Класс добавление контакта для gui"""

    def __init__(self, client_addresser, data_base):
        super().__init__()
        self.data_base = data_base
        self.client_addresser = client_addresser
        self.client_logger = getLogger('client')

        self.setFixedSize(350, 120)
        self.setWindowTitle('Выбор контакта для добавления:')
        # удаление диалога, если окно было закрыто преждевременно
        self.setAttribute(Qt.WA_DeleteOnClose)
        # сделать окно поверх других окон
        self.setModal(True)

        self.selector_label = QLabel('Выбор контакта для добавления:', self)
        self.selector_label.setFixedSize(200, 20)
        self.selector_label.move(10, 0)

        self.selector = QComboBox(self)
        self.selector.setFixedSize(200, 20)
        self.selector.move(10, 30)

        self.refresh_button = QPushButton('Обновление списка', self)
        self.refresh_button.setFixedSize(126, 30)
        self.refresh_button.move(60, 60)

        self.ok_button = QPushButton('Добавить контакт', self)
        self.ok_button.setFixedSize(116, 30)
        self.ok_button.move(230, 20)

        self.cancel_button = QPushButton('Отменить', self)
        self.cancel_button.setFixedSize(100, 30)
        self.cancel_button.move(230, 60)
        self.cancel_button.clicked.connect(self.close)

        # заполнить список возможных контактов
        self.potential_contacts_update()
        # назначить действие на кнопку обновить
        self.refresh_button.clicked.connect(self.update_potential_contacts)

    def potential_contacts_update(self):
        """Метод класса для заполнения списка всех возможных контактов"""
        self.selector.clear()
        # множество всех контактов и контактов клиента
        contacts_list = set(self.data_base.get_user_contacts_id())
        users_list = set(self.data_base.get_users())
        # удалим из списка самого себя
        users_list.remove(self.client_addresser.name_client)
        # добавляем список возможных контактов
        self.selector.addItems(users_list - contacts_list)

    def update_potential_contacts(self):
        """Метод класса для обновления известных пользоватей и обновления предполагаемых контактов"""
        try:
            self.client_addresser.user_list_request()
        except OSError:
            pass
        else:
            self.client_logger.debug('Выполнено обновление списка пользователя с сервера')
            self.potential_contacts_update()



from base64 import b64decode, b64encode
from json import JSONDecodeError
from sys import path as sys_path
from Cryptodome.PublicKey import RSA
from common.config_path_file import CONFIG_PATH
sys_path.append('../')
from PyQt5.QtCore import Qt, pyqtSlot
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor
from client_pack.main_window_conv import Ui_MainClientWindow
from PyQt5.QtWidgets import QMainWindow, qApp, QMessageBox, QApplication
from sys import argv, exit
from client_pack.add_contact import AddContactDialog
from client_pack.remove_contact import RemoveContactDialog
from common.erros import ServerError
from common.utils import client_log_config, read_conf
from logging import getLogger
from Cryptodome.Cipher import PKCS1_OAEP


client_logger = getLogger('client')
conf = read_conf(CONFIG_PATH)


class ClientMainWindow(QMainWindow):
    """Класс основного окна клиента"""
    def __init__(self, data_base, client_addresser, keys):
        super().__init__()
        self.data_base = data_base
        self.client_addresser = client_addresser

        self.select_dialog = None
        self.remove_dialog = None

        self.current_chat_key = None
        self.decrypter = PKCS1_OAEP.new(keys)
        self.cryptographer = None

        # загрузка конфигурации главного окна
        self.ui = Ui_MainClientWindow()
        self.ui.setupUi(self)

        # кнопка - выход
        self.ui.menu_exit.triggered.connect(qApp.exit)

        # кнопка - отправить сообщение
        self.ui.btn_send.clicked.connect(self.send_message)

        # кнопка - добавить контакт
        self.ui.btn_add_contact.clicked.connect(self.add_contact_window)
        self.ui.menu_add_contact.triggered.connect(self.add_contact_window)

        # кнопка - удалить контакт
        self.ui.btn_remove_contact.clicked.connect(self.del_contact_window)
        self.ui.menu_del_contact.triggered.connect(self.del_contact_window)

        # доп атрибуты
        self.contacts_model = None
        self.history_model = None
        self.messages = QMessageBox()
        self.current_chat = None  # Текущий контакт для обмена сообщениями
        self.ui.list_messages.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.ui.list_messages.setWordWrap(True)

        self.ui.list_contacts.doubleClicked.connect(self.select_act_user)
        self.clients_list_update()
        self.set_disabled_input()
        self.show()

    def send_message(self):
        """Метод класса для отправки сообщения пользователю"""
        message_text = self.ui.text_message.toPlainText()
        self.ui.text_message.clear()
        if not message_text:
            return

        # шифрование сообщения ключом получателя и упаковка в base64
        message_text_encrypt = self.cryptographer.encrypt(message_text.encode('utf8'))
        message_text_encrypt_base64 = b64encode(message_text_encrypt)

        try:
            self.client_addresser.send_message(self.current_chat, message_text_encrypt_base64.decode('ascii'))
        except ServerError as err:
            self.messages.critical(self, 'Ошибка', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Ошибка', 'Было потеряно соединение с сервером!')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
        except (ConnectionResetError, ConnectionAbortedError):
            self.messages.critical(self, 'Ошибка', 'Было потеряно соединение с сервером!')
            self.close()
        else:
            self.data_base.save_message(self.current_chat, 'out', message_text)
            client_logger.debug(f'Отправлено сообщение для {self.current_chat}: {message_text}')
            self.history_list_updater()

    def add_contact(self, new_contact):
        """Метод класса для добавления контакта в бд"""
        try:
            self.client_addresser.add_contact(new_contact)
        except ServerError as err:
            self.messages.critical(self, 'Ошибка сервера', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Ошибка', 'Было потеряно соединение с сервером!')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
        else:
            self.data_base.add_contact(new_contact)
            new_contact = QStandardItem(new_contact)
            new_contact.setEditable(False)
            self.contacts_model.appendRow(new_contact)
            client_logger.info(f'Контакт {new_contact} был добавленю')
            self.messages.information(self, 'Успешно', 'Контакт был добавлен.')

    def add_cont_action(self, item):
        """Метод класса обработчик добавления контактов"""
        new_contact = item.selector.currentText()
        self.add_contact(new_contact)
        item.close()

    def add_contact_window(self):
        """Метод класса для добавления контакта gui"""
        self.select_dialog = AddContactDialog(self.client_addresser, self.data_base)
        self.select_dialog.ok_button.clicked.connect(lambda: self.add_cont_action(self.select_dialog))
        self.select_dialog.show()

    def del_contact(self, item):
        """Метод класса для удаления контакта"""
        selected = item.selector.currentText()
        selected_id = self.data_base.get_user_cont_id(selected)

        try:
            self.client_addresser.dell_contact(selected)
        except ServerError as err:
            self.messages.critical(self, 'Ошибка сервера', err.text)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Ошибка', 'Было потеряно соединение с сервером!')
                self.close()
            self.messages.critical(self, 'Ошибка', 'Таймаут соединения!')
        else:
            self.data_base.delete_contact(selected_id)
            self.clients_list_update()
            client_logger.info(f'Контакт {selected} был удалён')
            self.messages.information(self, 'Успешно', 'Контакт был удалён.')
            item.close()
            # Если удалён активный пользователь, то деактивируем поля ввода.
            if selected == self.current_chat:
                self.current_chat = None
                self.set_disabled_input()

    def del_contact_window(self):
        """Метод класса для удаления контакта gui"""
        self.remove_dialog = RemoveContactDialog(self.data_base)
        self.remove_dialog.ok_button.clicked.connect(lambda: self.del_contact(self.remove_dialog))
        self.remove_dialog.show()

    def select_act_user(self):
        """Метод класса обработчик для выбора контакта двойным нажатием"""
        self.current_chat = self.ui.list_contacts.currentIndex().data()
        self.set_act_user()

    def set_act_user(self):
        """Метод класса для выбора активного пользователя"""
        # запрос публичного ключа пользователя и создания объекта шифрования
        try:
            self.current_chat_key = self.client_addresser.key_pub_request(self.current_chat)
            client_logger.debug(f'Загружен публичный ключ для {self.current_chat}')
            if self.current_chat_key:
                self.cryptographer = PKCS1_OAEP.new(RSA.import_key(self.current_chat_key))
        except (OSError, JSONDecodeError):
            self.current_chat_key = None
            self.cryptographer = None
            client_logger.debug(f'Ключ не был получен для {self.current_chat}')

        # если ключ не был найден
        if not self.current_chat_key:
            self.messages.warning(self, 'Ошибка!', 'Нет ключа шифрования для выбронного вами пользователя!.')
            return

        # ставит надпись и активирует кнопки
        self.ui.label_new_message.setText(f'Введите сообщение для {self.current_chat}:')
        self.ui.btn_clear.setDisabled(False)
        self.ui.btn_send.setDisabled(False)
        self.ui.text_message.setDisabled(False)
        # Заполняем окно историю сообщений по требуемому пользователю.
        self.history_list_updater()

    def clients_list_update(self):
        """Метод класса для обновления листа контактов"""
        contacts_list = self.data_base.get_user_contacts()
        self.contacts_model = QStandardItemModel()
        for i in sorted(contacts_list):
            item = QStandardItem(i)
            item.setEditable(False)
            self.contacts_model.appendRow(item)
        self.ui.list_contacts.setModel(self.contacts_model)

    def set_disabled_input(self):
        """Метод класса для выбора получателя сообщения"""
        self.ui.label_new_message.setText('Для выбора получателя нужно клинуть на нём дважды в окене контактов')
        self.ui.text_message.clear()
        if self.history_model:
            self.history_model.clear()

        # Поле ввода и кнопки не активны.
        self.ui.btn_clear.setDisabled(True)
        self.ui.btn_send.setDisabled(True)
        self.ui.text_message.setDisabled(True)

        self.current_chat = None
        self.current_chat_key = None
        self.cryptographer = None

    def history_list_updater(self):
        """Метод класса заполняющий историю сообщений"""
        list_messages = sorted(self.data_base.get_history_messages(self.current_chat), key=lambda item: item[3])
        if not self.history_model:
            self.history_model = QStandardItemModel()
            self.ui.list_messages.setModel(self.history_model)
        self.history_model.clear()
        length = len(list_messages)
        counter = 0

        if length > 22:
            counter = length - 20

        for i in range(counter, length):
            item = list_messages[i]
            if item[1] == 'in':
                mess = QStandardItem(f'Входящее сообщение от {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setBackground(QBrush(QColor(255, 213, 213)))
                mess.setTextAlignment(Qt.AlignLeft)
                self.history_model.appendRow(mess)
            elif item[1] == 'out':
                mess = QStandardItem(f'Исходящее сообщение от {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setTextAlignment(Qt.AlignRight)
                mess.setBackground(QBrush(QColor(204, 255, 204)))
                self.history_model.appendRow(mess)
        self.ui.list_messages.scrollToBottom()

    @pyqtSlot(dict)
    def message_receiver(self, message):
        """Метод класса проверяющий входящие сообщения, дешефрует сообщения, сохраняет в историю сообщений"""
        encrypt_message = b64decode(message[conf['MESS_TEXT']])  # строка байт
        try:
            decrypt_message = self.decrypter.decrypt(encrypt_message)  # расшифрованная строка байт
        except (ValueError, TypeError):
            self.messages.warning(self, 'Ошибка!', 'Сообщение не удалось декодировать!.')
            return

        sender = message[conf['ADDRESSER']]

        if sender == self.current_chat:
            # расшифрованная строка
            self.data_base.save_message(self.current_chat, 'in', decrypt_message.decode('utf8'))
            self.history_list_updater()
        else:
            #  проверка есть ли пользователь в контактах
            if self.data_base.checker_contact(sender):
                if self.messages.question(self, 'Новое сообщение', f'Вам пришло новое сообщение от {sender}, '
                                          f'открыть чат с ним?', QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
                    self.current_chat = sender
                    self.data_base.save_message(self.current_chat, 'in', decrypt_message.decode('utf8'))
                    self.set_act_user()
            else:
                if self.messages.question(self, 'Новое сообщение', f'Вам пришло новое сообщение от {sender}.\n '
                                          f'Данного пользователя нет в ваших контактах.\n'
                                          f' Добавить в контакты и открыть чат с ним?',
                                          QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
                    self.add_contact(sender)
                    self.current_chat = sender
                    self.data_base.save_message(self.current_chat, 'in', decrypt_message.decode('utf8'))
                    self.set_act_user()

    @pyqtSlot()
    def connect_lost(self):
        """Метод класса, отвечающий за потерю соединения"""
        self.messages.warning(self, 'Сбой соединения!', 'Было потеряно соединение с сервером!')
        self.close()

    @pyqtSlot()
    def signal_205(self):
        """Метод класса слот, обнавляющий бд по команде сервера"""
        if self.current_chat and not self.data_base.checker_user(self.current_chat):
            self.messages.warning(self, 'Сожалеем', 'Ваш собеседник был удалён с сервера.')
            self.set_disabled_input()
            self.current_chat = None
        self.clients_list_update()

    def do_connection(self, transport):
        """Метод класса, отвечающий за соединение"""
        transport.new_message.connect(self.message_receiver)
        transport.connect_lost.connect(self.connect_lost)
        transport.mess_205.connect(self.signal_205)


if __name__ == '__main__':
    from client_storage import ClientStorage
    from os.path import dirname, realpath

    app = QApplication(argv)
    dir_path = dirname(realpath(__file__))
    database = ClientStorage(dir_path, 'test_1')

    class Test1:
        client_name = 'test_1'

    window = ClientMainWindow(database, Test1, None)
    exit(app.exec_())



from logging import getLogger
from logs import server_log_config, client_log_config
from sys import exit
from common.utils import read_conf
from common.config_path_file import CONFIG_PATH


class PortDescriptor:
    """Класс для проверки порта"""
    def __init__(self, name_logger):
        self.conf = read_conf(CONFIG_PATH)
        self.name_logger = name_logger

        if self.name_logger == 'server':
            self.logger = getLogger('server')
        elif name_logger == 'client':
            self.logger = getLogger('client')

    def __set__(self, instance, value):
        if self.name_logger == 'server':
            self.name_logger = 'сервер'
        elif self.name_logger == 'client':
            self.name_logger = 'клиент'

        self.logger.debug(f'Получение порта для {self.name_logger}а.')
        if value < 0:
            self.logger.critical('Номер порта должен быть положительным числом!')
            exit(1)
        if value < 1024 or value > 65535:
            self.logger.critical('Номер порта должен быть указан в диапазоне от 1024 до 65535!')
            exit(1)
        instance.__dict__[self.port] = value
        if self.port == self.conf['PORT_DEF']:
            self.logger.info(f'Для работы {self.name_logger}а задан порт по умолчанию - {value}')
        else:
            self.logger.info(f'Получен порт для работы {self.name_logger}а - {value}')

    def __set_name__(self, owner, port):
        self.port = port



from common.utils import read_conf
from unittest import TestCase, main
from client import request_presence, response_analysis, get_answer, data_connect_serv
from time import ctime
from test_utils import TestSocket
from common.config_path_file import CONFIG_PATH


class TestClient(TestCase):
    """Тестовый класс для тестирования функций клиента"""
    config = read_conf(CONFIG_PATH)
    test_dict_get_ok = {
        config['RESPONSE']: 200
    }
    test_dict_send = {
        config['ACTION']: config['PRESENCE'],
        config['TIME']: ctime(),
        config['USER_NAME']: {
            config['ACCOUNT_NAME']: 'Test_name'
        }
    }
    test_dict_data_connect = {
        config['ADDR_SERV']: config['ADDR_DEF'],
        config['PORT_SERV']: config['PORT_DEF']
    }

    def test_request_presence_name(self):
        """Тестирует правильное ли по умолчанию название аккаунта вернёт функция request_presence()"""
        request_pres = request_presence(conf_name=CONFIG_PATH)
        self.assertEqual(request_pres[self.config['USER_NAME']]['account_name'], 'Guest')

    def test_response_analysis_ok(self):
        """Тестирует функцию response_analysis(), которая анализирует правильный ответ от сервера"""
        self.assertEqual(response_analysis(self.test_dict_get_ok, CONFIG_PATH), '200 : OK')

    def test_get_answer_ok(self):
        """Тестирует функцию ответа сервера в нужной кодировке"""
        test_sock_ok = TestSocket(self.test_dict_get_ok)
        self.assertEqual(get_answer(test_sock_ok, CONFIG_PATH), '200 : OK')

    def test_data_connect_serv_ok(self):
        """Тестирует данные для подключения на корректность"""
        self.assertEqual(data_connect_serv(CONFIG_PATH), self.test_dict_data_connect)


if __name__ == '__main__':
    main()



from sys import path as sys_path
sys_path.append('../')
from logging import getLogger, Formatter, StreamHandler, FileHandler, DEBUG
from os.path import abspath, dirname

logger = getLogger('client')

# объект для форматирования строки
format_message = Formatter('%(asctime)s - %(levelname)s - %(module)s - %(message)s')

# обработчик для логирования в файл
path_file = f'{dirname(abspath(__file__))}/client.log'
file_hand = FileHandler(path_file, encoding='utf-8')
file_hand.setLevel(DEBUG)
file_hand.setFormatter(format_message)

# добавили в логгер новый обработчик событий и установили уровень логирования
logger.addHandler(file_hand)
logger.setLevel(DEBUG)

if __name__ == '__main__':
    # тестовый запуск - вывод данных в консоль, которые логгер должен выдать в файл
    terminal = StreamHandler()
    terminal.setLevel(DEBUG)
    terminal.setFormatter(format_message)
    logger.addHandler(terminal)
    logger.setLevel(DEBUG)
    logger.debug('test debug')
    logger.info('test info')
    logger.warning('test warning')
    logger.error('test error')
    logger.critical('test critical')



from sys import path as sys_path
sys_path.append('../')
from logging import getLogger, Formatter, StreamHandler, handlers, FileHandler, DEBUG
from os.path import dirname, abspath
from time import strftime
import datetime


logger = getLogger('server')

# объект для форматирования строки
format_message = Formatter('%(asctime)s - %(levelname)s - %(module)s - %(message)s')

# обработчик для логирования в файл
path_dir = dirname(abspath(__file__))
path_file = f'{path_dir}/server.log'

file_hand = handlers.TimedRotatingFileHandler(
    path_file, 'D', 1, encoding='utf-8')  # ротация логов каждый день
file_hand.setLevel(DEBUG)
file_hand.setFormatter(format_message)

# добавили в логгер новый обработчик событий и установили уровень логирования
logger.addHandler(file_hand)
logger.setLevel(DEBUG)

date = datetime.datetime.now()
date = f"{date.year}-{strftime('%m')}-{strftime('%d')}"
file_hand_one_day = FileHandler(f'{path_dir}/server.log.{date}')
file_hand_one_day.setLevel(DEBUG)
file_hand_one_day.setFormatter(format_message)
logger.addHandler(file_hand_one_day)
logger.setLevel(DEBUG)


if __name__ == '__main__':
    # тестовый запуск - вывод данных в консоль, которые логгер должен выдать в файл
    terminal = StreamHandler()
    terminal.setLevel(DEBUG)
    terminal.setFormatter(format_message)
    logger.addHandler(terminal)
    logger.setLevel(DEBUG)
    logger.debug('test debug')
    logger.info('test info')
    logger.warning('test warning')
    logger.error('test error')
    logger.critical('test critical')



from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton, QApplication
from PyQt5.QtCore import Qt
from sys import argv
from client_pack.client_storage import ClientStorage


class RemoveContactDialog(QDialog):
    """Класс gui для удаления контакта"""
    def __init__(self, data_base):
        super().__init__()
        self.data_base = data_base

        self.setFixedSize(350, 120)
        self.setWindowTitle('Какой контакт удалить: ')
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)

        self.selector_label = QLabel('Выберите контакт: ', self)
        self.selector_label.setFixedSize(200, 20)
        self.selector_label.move(10, 0)

        self.selector = QComboBox(self)
        self.selector.setFixedSize(200, 20)
        self.selector.move(10, 30)
        self.selector.addItems(sorted(self.data_base.get_user_contacts()))

        self.ok_button = QPushButton('Удалить', self)
        self.ok_button.setFixedSize(100, 30)
        self.ok_button.move(230, 20)

        self.cancel_button = QPushButton('Отменить', self)
        self.cancel_button.setFixedSize(100, 30)
        self.cancel_button.move(230, 60)
        self.cancel_button.clicked.connect(self.close)


if __name__ == '__main__':
    my_app = QApplication(argv)
    database = ClientStorage('test_1')
    window = RemoveContactDialog(database)
    database.add_contact('test_1')
    database.add_contact('test_2')
    print(database.get_user_contacts())
    window.selector.addItems(sorted(database.get_user_contacts()))
    window.show()
    my_app.exec_()



#!../venv/bin/python3
from os import urandom
from os.path import dirname, realpath, exists
from sys import argv, exit
from Cryptodome.PublicKey.RSA import generate, import_key
from PyQt5.QtWidgets import QApplication
from client_pack.start_dialog import ClientNameDialog
from client_pack.transport import ClientTransport
from logging import getLogger
from client_pack.main_window import ClientMainWindow
from common.erros import ServerError
from client_pack.client_storage import ClientStorage
from common.config_path_file import CONFIG_CLIENT_PATH
from common.utils import check_port_argv, check_addr_argv, check_name_argv, check_passwd_argv, read_conf


client_logger = getLogger('client')
conf = read_conf(CONFIG_CLIENT_PATH)


def main():
    """Функция запуска клиента"""
    client_logger.debug('Получение корректных данных для соединения с сервером')
    addr_server, port_server, name_client, passwd_client = check_addr_argv(), check_port_argv(), check_name_argv(), \
        check_passwd_argv()
    # создание приложение клиент
    client_app = QApplication(argv)

    # если имя и/или пароль не указали запросим
    if not name_client or not passwd_client:
        start_dialog = ClientNameDialog()
        client_app.exec_()
        # если пользователь ввёл имя и нажал ОК, то сохраняем ведённое и удаляем объект.
        if start_dialog.ok_flag:
            name_client = start_dialog.client_name.text()
            passwd_client = start_dialog.client_passwd.text()
            del start_dialog
            client_logger.debug(f'Используется логин - {name_client}, пароль - {passwd_client}.')
        else:
            exit(0)

    client_logger.info(f'Старт работы клиента с параметрами: адрес сервера - {addr_server}, '
                       f'порт: {port_server}, имя пользователя {name_client}.')

    # читаем файл с ключом чтоб его получить, если его нет генерируется новая пара ключей
    dir_path = dirname(realpath(__file__))
    file_with_key = f'{dir_path}/{name_client}.key'
    if not exists(file_with_key):
        keys_rsa = generate(2048, urandom)
        with open(file_with_key, 'wb') as file:
            file.write(keys_rsa.export_key())
    else:
        with open(file_with_key, 'rb') as file:
            keys_rsa = import_key(file.read())

    client_logger.debug('Ключи успешно прочитаны.')

    data_base = ClientStorage(conf['DB_PATH'], name_client)

    # cоздание объекта транспорт и запуск транспортного потока
    transport = None
    try:
        transport = ClientTransport(port_server, addr_server, data_base, name_client, passwd_client, keys_rsa)
        transport.daemon = True
        transport.start()
    except ServerError as err:
        print(err.text)
        exit(1)

    # создаём GUI
    main_window = ClientMainWindow(data_base, transport, keys_rsa)
    main_window.do_connection(transport)
    main_window.setWindowTitle(f'Программа Чат alpha version - {name_client}')
    client_app.exec_()
    # при закрытом gui, закрываем транспорт
    transport.transport_close()
    transport.join()


if __name__ == '__main__':
    main()



from PyQt5.QtWidgets import QDialog, QPushButton, QLineEdit, QApplication, QLabel, qApp


class ClientNameDialog(QDialog):
    """Класс стартового диалога, для выбора имени пользователя"""
    def __init__(self):
        super().__init__()

        self.ok_flag = False
        self.setWindowTitle('Здравствуйте!')
        self.setFixedSize(175, 130)

        self.label = QLabel('Введите свое имя или ник: ', self)
        self.label.move(8, 10)
        self.label.setFixedSize(175, 10)

        self.client_name = QLineEdit(self)
        self.client_name.setFixedSize(154, 20)
        self.client_name.move(10, 30)

        self.label_passwd = QLabel('Введите пароль:', self)
        self.label_passwd.move(10, 55)
        self.label_passwd.setFixedSize(150, 15)

        self.client_passwd = QLineEdit(self)
        self.client_passwd.setFixedSize(154, 20)
        self.client_passwd.move(10, 75)
        self.client_passwd.setEchoMode(QLineEdit.Password)

        self.ok_button = QPushButton('Старт', self)
        self.ok_button.move(6, 100)
        self.ok_button.clicked.connect(self.click_ok)

        self.cancel_button = QPushButton('Выйти', self)
        self.cancel_button.move(90, 100)
        self.cancel_button.clicked.connect(qApp.exit)

        self.show()

    def click_ok(self):
        """Метод класса обработчик, следящий за вводом имени и нажатием кнопки ок"""
        if self.client_name.text() and self.client_passwd.text():
            self.ok_flag = True
            qApp.exit()


if __name__ == '__main__':
    my_app = QApplication(list())
    dial = ClientNameDialog()
    my_app.exec_()



from sys import path as sys_path
sys_path.append('../')
from os.path import dirname, realpath
from sqlalchemy import create_engine, Column, Integer, String, Text, DateTime
from sqlalchemy.orm import sessionmaker, declarative_base
from datetime import datetime
from common.utils import read_conf
from common.config_path_file import CONFIG_CLIENT_PATH


class ClientStorage:
    """Класс для работы с базой данных клиента в декларативном стиле"""
    Base = declarative_base()

    class FamousUsers(Base):
        """Класс известные для клиента пользователи"""
        __tablename__ = 'famous_users'
        id = Column(Integer, primary_key=True)
        login = Column(String, unique=True)

        def __init__(self, login):
            self.login = login
            super().__init__()

    class HistoryMessages(Base):
        """Класс история сообщений"""
        __tablename__ = 'history_messages'
        id = Column(Integer, primary_key=True)
        contact_login = Column(String)
        type_message = Column(String)
        message = Column(Text)
        date = Column(DateTime)

        def __init__(self, contact_login, type_message, message):
            self.contact_login = contact_login
            self.type_message = type_message
            self.message = message
            self.date = datetime.now()
            super().__init__()

    class UserContacts(Base):
        """Класс контакты пользователя"""
        __tablename__ = 'user_contacts'
        id = Column(Integer, primary_key=True)
        contact_login = Column(String, unique=True)

        def __init__(self, contact_login):
            self.contact_login = contact_login
            super().__init__()

    def __init__(self, path, name):
        # установка соединения с бд и сбор конф информации
        # echo=True - ведение лога, poll_recycle=7200 - переустановка соединения с бд каждые 2 часа
        self.engine = create_engine(f'sqlite:///{path}/{name}.db3', echo=True, pool_recycle=7200,
                                    connect_args={'check_same_thread': False})
        self.Base.metadata.create_all(self.engine)  # создаём все таблицы
        session_fabric = sessionmaker(bind=self.engine)
        self.session = session_fabric()  # создаём сессию

        self.session.query(self.UserContacts).delete()
        self.session.commit()

    def table_clear(self, command):
        """Метод класса для очищения таблиц от данных"""
        try:
            if command == 'all':
                self.session.query(self.FamousUsers).delete()
                self.session.query(self.HistoryMessages).delete()
                self.session.query(self.UserContacts).delete()
                self.session.commit()
            else:
                eval(f'self.session.query(self.{command}).delete()')
                self.session.commit()
        except (Exception, ) as err:
            print(f'Ошибка - {err} при работе с данными таблицы!')

    def add_contact(self, user):
        """Метод класса для добавления нового контакта"""
        try:
            if not self.session.query(self.UserContacts).filter_by(contact_login=user).count():
                new_contact = self.UserContacts(user)
                self.session.add(new_contact)
                self.session.commit()
        except (Exception, ) as err:
            print(f'Ошибка - {err} при работе с данными таблицы!')

    def delete_contact(self, user):
        """Метод класса для удаления контакта из контактов клиента"""
        try:
            del_contact = self.session.query(self.UserContacts).filter_by(contact_login=user)
            if del_contact.count():
                del_contact.delete()
                self.session.commit()
        except (Exception, ) as err:
            print(f'Ошибка - {err} при работе с данными таблицы!')

    def add_users(self, users_list):
        """Метод класса для добавления известных пользователей"""
        try:
            self.session.query(self.FamousUsers).delete()
            for user in users_list:
                if not self.session.query(self.FamousUsers).filter_by(login=user).count():
                    new_user = self.FamousUsers(user)
                    self.session.add(new_user)
                    self.session.commit()
        except (Exception, ) as err:
            print(f'Ошибка - {err} при работе с данными таблицы!')

    def save_message(self, from_user, for_user, message):
        """Метод класса для сохранения сообщения"""
        try:
            new_message = self.HistoryMessages(from_user, for_user, message)
            self.session.add(new_message)
            self.session.commit()
        except (Exception, ) as err:
            print(f'Ошибка - {err} при работе с данными таблицы!')

    def get_user_contacts_id(self):
        """Метод класса возвращает список контактов пользователя в виде id"""
        return [user[0] for user in self.session.query(self.UserContacts.contact_login).all()]

    def get_user_contacts(self):
        """Метод класса возвращает список контактов пользователя"""
        list_contact = list()
        contact_login_id_list = [user_id[0] for user_id in self.session.query(
            self.UserContacts.contact_login).all()]
        for user_id in contact_login_id_list:
            name_search = self.session.query(self.FamousUsers.login).filter_by(id=user_id)
            for name in name_search:
                list_contact.append(name[0])
        return list_contact

    def get_users(self):
        """Метод класса возращает список известных пользователей"""
        return [user[0] for user in self.session.query(self.FamousUsers.login).all()]

    def get_user_cont_id(self, contact_login):
        """Метод класса возвращает id контакта пользователя"""
        contact_login_id = [login[0] for login in self.session.query(
            self.FamousUsers.id).filter_by(login=contact_login)][0]
        return [user_id[0] for user_id in self.session.query(
            self.UserContacts.contact_login).filter_by(contact_login=contact_login_id)][0]

    def checker_user(self, login):
        """Метод класса для проверки известный ли клиенту пользователь"""
        if self.session.query(self.FamousUsers).filter_by(login=login).count():
            return True
        else:
            return False

    def checker_contact(self, contact_login):
        """Метод класса проверяет есть ли контакт в контактах"""
        contact_login_id = self.session.query(self.FamousUsers.id).filter_by(login=contact_login)
        if self.session.query(self.UserContacts).filter_by(contact_login=contact_login_id).count():
            return True
        else:
            return False

    def get_history_messages(self, contact):
        """Метод класса возращает историю сообщений"""
        history = self.session.query(self.HistoryMessages).filter_by(contact_login=contact)
        return [(history_line.contact_login, history_line.type_message, history_line.message, history_line.date)
                for history_line in history.all()]


if __name__ == '__main__':
    dir_path = dirname(realpath(__file__))
    client_storage = ClientStorage(dir_path, 'cl1')
    client_storage.table_clear('all')
    client_storage = ClientStorage(dir_path, 'cl2')
    client_storage.table_clear('all')
    # client_storage.add_contact('bot628')
    # client_storage.add_contact('botT1000')
    # client_storage.add_contact('WalleT34')
    # client_storage.delete_contact('bot628')
    # print(client_storage.get_user_contacts())
    # client_storage.add_users(['bot628', 'botT1000', 'WalleT34'])
    # print(client_storage.get_users())
    # client_storage.save_message('WalleT34', 'botT1000', 'Hello botT1000')
    # print(client_storage.checker_contact('botT1000'))
    # print(client_storage.get_history_messages(from_who='WalleT34'))



from setuptools import setup, find_packages

setup(name='robot_messenger_client',
      version='0.1.0',
      description='Client for messenger',
      author='Eugene Kaddo',
      author_email='message.chaos628@gmail.com',
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )


from sys import argv, path as sys_path
sys_path.append('../')
from common.utils import read_conf
from socket import socket
from time import ctime
from inspect import stack
from logging import getLogger
from logs import server_log_config, client_log_config
from common.config_path_file import CONFIG_PATH


logger = None
module = argv[0].split('/')[-1]
conf = read_conf(CONFIG_PATH)

if 'server' in module:
    logger = getLogger('server')
elif 'client' in module:
    logger = getLogger('client')


def log(func):
    """Функция логирования"""

    def wrapper(*args, **kwargs):
        func_call = func(*args, **kwargs)
        date = ctime()
        call_from = stack()[1][3]
        logger.debug(f'{date} - функция {func.__name__} вызвана из функции {call_from} из модуля {func.__module__}')
        return func_call

    return wrapper


def login_needed(func):
    """Функция декоратор, проверяющая, что клиент авторизован на сервере"""
    def check(*args, **kwargs):
        from server_pack.kernel import MessageReceiver
        if isinstance(args[0], MessageReceiver):
            found = False
            for arg in args:
                if isinstance(arg, socket):
                    for client in args[0].client_names:
                        if args[0].client_names[client] == arg:
                            found = True
            # Если presence сообщение, то разрешено
            for arg in args:
                if isinstance(arg, dict):
                    if conf['ACTION'] in arg and arg[conf['ACTION']] == conf['PRESENCE']:
                        found = True
            if not found:
                raise TypeError
        return func(*args, **kwargs)
    return check
